lm.component('iotThings', {
  templateUrl: 'components/iotThings.html',
  bindings: {
	    // hero: '='
  },
  controller: function($element, $http, $modal,serviceLimboMedia,serviceNotification) {
		var ctrl = this;
		ctrl.sensors = [];
		
		ctrl.daterange = {
			from: moment().subtract(1,'days'),
			to: moment().add(1,'hour')
		}
		
		ctrl.sensorsString = [];
		ctrl.sensorsCoordinate = [];
		
		ctrl.loadSensors = function() {
			$http.get(lmBase.url + 'iot/items')
				.success(function(data) {
					for(x in data) {
						var it = data[x];
						
						/* initially checked */
						for(y in ctrl.settings.sensorsOn) {
							var sensorid = ctrl.settings.sensorsOn[y];
							if(sensorid == it.id) {
								it.checked = true;
							}
						}
						
						if(it.type == 'CONTROL_SELECT') {
							for(y in it.options) {
								var option = it.options[y];
								if(it.last && it.last.val == option.key) {
									it.selectedOption = option; 
								}
							}
						}
						
					}					
					ctrl.sensors = data;
					ctrl.redrawAll();
			}).error(serviceLimboMedia.handleHttpError);
		};
		
		ctrl.sensorEdit = function(sens,sts) {
			var modalInstance = $modal.open({
			      templateUrl: 'dialogs/dialogThingCrup.html',
			      controller: 'CtrlDialogThingCrup',
			      resolve: {
			        item: function () {
			          return sens;
			        }
			      }
			    });
			modalInstance.result.then(ctrl.loadSensors, function () { /* dismissed */ });
		};	
		
		ctrl.sensorDelete = function(sens) {
			var modalInstance = $modal.open({
			      templateUrl: 'dialogs/dialogThingDelete.html',
			      controller: 'CtrlDialogThingDelete',
			      resolve: {
			    	  sensor: function () {
			          return sens;
			        }
			      }
			    });
			modalInstance.result.then(ctrl.loadSensors, function () { /* dismissed */ });		 
		};
		
		ctrl.sensorPush = function(sens) {
			var modalInstance = $modal.open({
			      templateUrl: 'dialogs/dialogThingPush.html',
			      controller: 'CtrlDialogThingPush',
			      resolve: {
			    	  sensor: function () {
			          return sens;
			        }
			      }
		    });
		};
		
		ctrl.sensorPull = function(it) {
			var modalInstance = $modal.open({
			      templateUrl: 'dialogs/dialogThingPull.html',
			      controller: 'CtrlDialogThingPull',
			      resolve: {
			        item: function () {
			          return it;
			        }
			      }
			    });
			modalInstance.result.then(ctrl.loadSensors, function () { /* dismissed */ });
		};	
		
		ctrl.sensorShare = function(sens) {
			var modalInstance = $modal.open({
			      templateUrl: 'dialogs/dialogShareSimple.html',
			      controller: 'CtrlDialogShareSimple',
			      resolve: {
			        user: function () { return sens.x.user; },
				    url: function () { return lmBase.url + 'iot/item/' + sens.id + '/shares'; }
			      }
			    });
			modalInstance.result.then(ctrl.loadSensors, function () { /* dismissed */ });
		};
		
		ctrl.openAction = function(it) {
			var modalInstance = $modal.open({
			      templateUrl: 'dialogs/dialogThingAction.html',
			      controller: 'CtrlDialogThingAction',
			      resolve: {
			        item: function () {
			          return it;
			        }
			      }
			    });
			modalInstance.result.then(ctrl.loadControls, function () { /* dismissed */ });
		};	
		
		ctrl.sensorLoadData = function(sens) {
			var par = {
				from: ctrl.daterange.from.valueOf(),
				to: ctrl.daterange.to.valueOf()
			}
			$http.get(lmBase.url + 'iot/item/' + sens.id + '/rawdata',{params : par, targetSENS: sens})
				.success(function(data, status, headers, config) {
					config.targetSENS.data = data;
					ctrl.redraw();
			}).error(serviceLimboMedia.handleHttpError);
		}
		
		ctrl.redrawAll = function() {
			for(x in ctrl.sensors) {
				var sens = ctrl.sensors[x];
				if(sens.checked) {
					ctrl.sensorLoadData(sens);
				}
			}			
		}
		
		ctrl.toggleSensor = function(sens) {
			sens.checked = !sens.checked;
			if(sens.checked) {
				ctrl.sensorLoadData(sens);
			}
			else {
				sens.data = null;
				ctrl.redraw();
			}
			ctrl.saveToggled();
		};		
		
		ctrl.redraw = function() {
			ctrl.sensorsString = [];
			ctrl.sensorsCoordinate = [];
			
			var sd = [];
			var yaxises = [];
			
			var axcount = 1;
			for(x in ctrl.sensors) {
				var sens = ctrl.sensors[x];
				if(sens.data) {
					if(sens.dataType == 'STRING') {
						ctrl.sensorsString.push(sens);
					}
					else if(sens.dataType == 'COORDINATE') {
						ctrl.sensorsCoordinate.push(sens);
					}
					else {
						var axisToUse = null;
						for(y in yaxises) {
							var axis = yaxises[y];
							if(('BOOLEAN' == axis.dataType && 'BOOLEAN' == sens.dataType) || axis.name == sens.unit) {
								axisToUse = axis;
							}
						}
						if(!axisToUse) {
							axisToUse = {
								name: sens.unit,
								dataType: sens.dataType,
								idx: axcount++
							}
							yaxises.push(axisToUse);
						}
						
						var graphdata = {
								data: sens.data,
								label: sens.name + ' ' + sens.unit,
								yaxis: axisToUse.idx
						};

						if("FIRE" == sens.dataType) {
							graphdata = {
									data: sens.data,
									label: sens.name + ' ' + sens.unit,
									points: { show: true },
									yaxis: axisToUse.idx
							};
						}
						
						sd.push(graphdata);
					}
				}
			}
			var el = $($element).find('.plotit');
			var graph = $.plot(el, sd, {
				legend: {show:true, position:"nw",noColumns:1},
				lines: {show:true},
				xaxis: {
					mode: "time",
					timezone: "browser",
					timeformat: "%Y-%m-%d<br/>%H:%M:%S"
				},
				yaxes: yaxises,
				selection: { mode: "x" },
				tooltip: true,
				tooltipOpts: { content: "<h4>%s</h4> Value: %y.3<br/>%x" },
				grid: { hoverable: true }				
			});			
			
		};
		
		ctrl.saveToggled = function() {
			ctrl.settings.sensorsOn = [];
			for(x in ctrl.sensors) {
				var it = ctrl.sensors[x];
				if(it.checked) {
					ctrl.settings.sensorsOn.push(it.id);
				}
			}
			
			ctrl.saveSettings();
		}
		
		/* CONTROLS */
		ctrl.controlDoFire = function(con) {
			$http.post(lmBase.url + 'iot/item/' + con.id + '/control', {})
				.success(function(data) {
					serviceNotification.ok('DONE');
					con.last = data;
				}).error(serviceLimboMedia.handleHttpError);		
		};
		ctrl.controlDoSwitch = function(con, valOn) {
			var req = {val: valOn};
			$http.post(lmBase.url + 'iot/item/' + con.id + '/control', req)
				.success(function(data) {
					con.last = data;
				}).error(serviceLimboMedia.handleHttpError);		
		};

		ctrl.controlDoSelect = function(con, valOn) {
			var req = {val: valOn.key};
			$http.post(lmBase.url + 'iot/item/' + con.id + '/control', req)
				.success(function(data) {
					serviceNotification.ok('DONE');
					con.last = data;
				}).error(serviceLimboMedia.handleHttpError);		
		};

		ctrl.onPercentSliderChange = function(con, val) {
			var req = {'val': val};
			$http.post(lmBase.url + 'iot/item/' + con.id + '/control', req)
				.success(function(data) {
					serviceNotification.ok('DONE');
					con.last = data;
			}).error(serviceLimboMedia.handleHttpError);		
		}

		ctrl.controlDoNumber = function(con) {
			if(!(con.last && con.last.val))
				return;
			
			var req = {val: con.last.val};
			$http.post(lmBase.url + 'iot/item/' + con.id + '/control', req)
				.success(function(data) {
					serviceNotification.ok('DONE');
					con.last = data;
				}).error(serviceLimboMedia.handleHttpError);		
		};

		ctrl.controlDoString = function(con) {
			if(!(con.last && con.last.val))
				return;
			var req = {val: con.last.val};
			$http.post(lmBase.url + 'iot/item/' + con.id + '/control', req)
				.success(function(data) {
					serviceNotification.ok('DONE');
					con.last = data;
				}).error(serviceLimboMedia.handleHttpError);		
		};
		
		
		/* SETTINGS */
		ctrl.settings = new Object();

		ctrl.saveSettings = function() {
			serviceLimboMedia.saveLocalSettings('iot');
		};	
		
		ctrl.settings = serviceLimboMedia.getLocalSettings('iot');
		if(ctrl.settings.sensorsOn == null) ctrl.settings.sensorsOn = [];
		ctrl.loadSensors();
		
		
	}
});
