lm.component('pageFiles', {
  templateUrl: 'components/pageFiles.html',
  bindings: {
  },
  $canActivate: function($nextInstruction, $prevInstruction, serviceLimboMedia) {
	  return serviceLimboMedia.isAuthenticated();
  },
  controller: function($scope, $http, $timeout, $modal, serviceNotification, serviceLimboMedia, serviceMediaPlayer, serviceUpload, serviceFullscreen, serviceClipboard) {
	  var ctrl = this;
	  
	  ctrl.loadFolderID = 0;
	  
		/* ========== */
		/* = SEARCH = */
		/* ========== */	
		ctrl.search = {active: false, value: ''};

		/* ======== */
		/* = VIEW = */
		/* ======== */
		ctrl.view = 'LIST';

		ctrl.toggleView = function() {
			if(ctrl.view == 'LIST') {
				ctrl.view = 'GALLERY';
				ctrl.predicate = ctrl.settings.gallery.order;
			}
			else if(ctrl.view == 'GALLERY') {
				ctrl.view = 'AUDIO';
				ctrl.predicate = ctrl.settings.audio.order;
			}
			else if(ctrl.view == 'AUDIO') {
				ctrl.view = 'LIST';
				ctrl.predicate = ctrl.settings.list.order;
			}
			
			ctrl.settings.view = ctrl.view;
			ctrl.saveSettings();
			ctrl.filesLoad();
		};
		
		/* =============================== */
		/* = CURRENT FOLDER / SUBFOLDERS = */
		/* =============================== */
		
		ctrl.folder = null;				/* current folder */
		ctrl.folderStack = new Array();	/* folder stack (for breadcrumb) */
		ctrl.folders = new Array();		/* list of subfolders */
		
		
		ctrl.folderLoad = function(folderID) {
			ctrl.folder = null;
			ctrl.folderStack = new Array();
			ctrl.folders = new Array();
			
			$http.get(lmBase.url + 'f/folder/' + folderID + '/stacklist')
				.success(function(data) {
				// stack
				for(x in data.stack) {
					var it = data.stack[x];
					ctrl.folderStack.push(it);
					if(x == data.stack.length - 1) {
						ctrl.folder = it;
					}
				}
				
				// items (subfolders)
				for(x in data.items) {
					var it = data.items[x];
					ctrl.folders.push(it);
				}
				
				ctrl.filesLoad();		
			}).error(serviceLimboMedia.handleHttpError);
			
		};
		
		ctrl.folderReload = function() {
			ctrl.folderLoad(ctrl.folder == null ? 0 : ctrl.folder.id);
		};

		/* ================= */
		/* = CUT AND PASTE = */
		/* ================= */
		
		ctrl.clipboard = serviceClipboard;

		ctrl.clipCopy = function() {
			if(serviceClipboard.items.length == 0) {
				serviceClipboard.working = false;
				ctrl.folderReload();
				return;
			}
			serviceClipboard.working = true;
			
			var tomove = serviceClipboard.items.pop();
			
			var req = {target: ctrl.folder.id};

			var earl = lmBase.url + (tomove.type == null ? 'f/folder/' : 'f/file/') + tomove.id + '/copy';
				$http.post(earl,req)
					.success(function(data) {
						ctrl.clipCopy();
					})
					.error(function(data,status) {
						serviceLimboMedia.handleHttpError(data,status);
						ctrl.clipCopy();
					});
		};
		
		ctrl.clipMove = function() {
			if(serviceClipboard.items.length == 0) {
				serviceClipboard.working = false;
				ctrl.folderReload();
				return;
			}
			serviceClipboard.working = true;
			
			var tomove = serviceClipboard.items.pop();
			
			var req = {target: ctrl.folder.id};
			
			var earl = lmBase.url + (tomove.type == null ? 'f/folder/' : 'f/file/') + tomove.id + '/move';
				$http.post(earl,req)
					.success(function(data) {
						ctrl.clipMove();
					})
					.error(function(data,status) {
						serviceLimboMedia.handleHttpError(data,status);
						ctrl.clipMove();
					});
		};
		
		/* ============== */
		/* = FOLDER OPS = */
		/* ============== */	
		
		ctrl.folderDelete = function(valueItem) {
			var modalInstance = $modal.open({
			      templateUrl: 'dialogs/dialogFolderDelete.html',
			      controller: 'CtrlDialogFolderDelete',
			      resolve: {
			        folder: function () { return valueItem; }
			      }
			    });
			 
			modalInstance.result.then(
				function () { ctrl.folderReload(); },
			    function () { /* dismissed */ }
			);
		};
		
		ctrl.folderShares = function(valueItem) {
			var modalInstance = $modal.open({
			      templateUrl: 'dialogs/dialogShare.html',
			      controller: 'CtrlDialogShare',
			      resolve: {
			        user: function () {
				          return valueItem.x.user;
				        },
				    url: function () {
				    	return lmBase.url + 'f/folder/' + valueItem.id + '/shares';
				    }
			      }
			    });
			 
			modalInstance.result.then(
				function () { ctrl.folderReload(); },
			    function () { ctrl.folderReload(); }
			);
		};
		
		ctrl.folderTokens = function(valueItem) {
			var modalInstance = $modal.open({
			      templateUrl: 'dialogs/dialogFolderTokens.html',
			      controller: 'CtrlDialogFolderTokens',
			      resolve: {
			        id: function () {
			        	return valueItem.id;
			        }
			      }
			    });
			 
			modalInstance.result.then(
				function () { ctrl.folderReload(); },
			    function () { ctrl.folderReload(); }
			);
		};		
		
		ctrl.folderEdit = function(valueItem) {
			var modalInstance = $modal.open({
			      templateUrl: 'dialogs/dialogRename.html',
			      controller: 'CtrlDialogRename',
			      resolve: {
				    id: function () { return valueItem.id; },
				    url: function () { return lmBase.url + 'f/folder/' + valueItem.id; },
			        name: function () { return valueItem.name; }		
			      }
			    });
			 
			modalInstance.result.then(
				function () { ctrl.folderReload(); },
			    function () { /* dismissed */ }
			);
		};	
		
		ctrl.folderSynchronize = function(valueFolder) {
			$http.post(lmBase.url + 'f/folder/' + valueFolder.id + '/sync')
				.success(function(data) {
					serviceNotification.ok('MSG_JOB_STARTED');
			}).error(serviceLimboMedia.handleHttpError);
		};
		
		ctrl.folderMount = function() {
			var modalInstance = $modal.open({
			      templateUrl: 'dialogs/dialogFolderMount.html',
			      controller: 'CtrlDialogFolderMount',
			      resolve: {}
			    });
			 
			modalInstance.result.then(
				function () { ctrl.folderReload(); },
			    function () { /* dismissed */ }
			);
		};
		
		
		/* ========= */
		/* = FILES = */
		/* ========= */
		ctrl.files = new Array();

		/* paging & sorting */
		ctrl.fr = 0;
		ctrl.hasMore = false;
		ctrl.predicate = 'NAME';
		ctrl.dir = 'asc';	
		
		ctrl.filesClear = function() {
			ctrl.files = new Array();
			ctrl.fr = 0;
			ctrl.hasMore = false;
		};	
		
		ctrl.filesLoad = function() {
			ctrl.filesClear();
			ctrl.filesMore();
		};
		
		ctrl.filesMore = function(loadAll) {
			var par = new Object();
			par.fr = ctrl.fr;
			ctrl.fr += ctrl.settings.pageSize;
			if(!loadAll) {
				par.mr = ctrl.settings.pageSize;
			}
			par.order = ctrl.predicate;
			par.dir = ctrl.dir;
			par.details = false;
			
			if('AUDIO' == ctrl.view) par.fileType = 'AUDIO';
			
			if(ctrl.search.active) {
				par.s = ctrl.search.value;
				if(ctrl.folder != null) {
					par.folder = ctrl.folder.id;
				}
				
				$http.get(lmBase.url + 'f/files/search',{params : par})
				.success(function(data) {
					for(x in data) {
						var it = data[x];
						lmBase.pimpLmFile(it);
						ctrl.files.push(it);
					}
					ctrl.hasMore = data.length == ctrl.settings.pageSize;
				}).error(serviceLimboMedia.handleHttpError);
			}
			else {
				if(ctrl.folder == null) return;
				
				par.folder = ctrl.folder.id;
				
				$http.get(lmBase.url + 'f/files/list',{params : par})
					.success(function(data) {
						for(x in data) {
							var it = data[x];
							lmBase.pimpLmFile(it);
							ctrl.files.push(it);
						}
						ctrl.hasMore = data.length == ctrl.settings.pageSize;
					}).error(serviceLimboMedia.handleHttpError);
			}
		};
		
		ctrl.changeOrder = function(valueOrder) {
			if(ctrl.predicate == valueOrder) {
				ctrl.dir = ctrl.dir == 'asc' ? 'desc' : 'asc';
			}
			else {
				ctrl.dir = 'asc';
				ctrl.predicate = valueOrder;
			}
			ctrl.filesLoad();
		};	
		
		ctrl.deleteFile = function(valueItem) {
			$http.delete(lmBase.url + 'f/file/' + valueItem.id)
				.success(function(data) {
					for(x in ctrl.files) {
						if(ctrl.files[x].id == valueItem.id) {
							ctrl.files.splice(x,1);
							break;
						}
					}
				}).error(serviceLimboMedia.handleHttpError);				
		};
		
		ctrl.selectionToggle = function() {
			if(ctrl.files.length > 0) {
				var targetState = !ctrl.files[0].client.select;
				for(x in ctrl.files) {
					ctrl.files[x].client.select = targetState;
				}
			}
		};
		
		ctrl.selectionDownload = function() {
		    var ids = new Array();
		    for(x in ctrl.files) {
				if(ctrl.files[x].client.select) {
					ids.push(ctrl.files[x]);
				}
			}
			
		    if(ids.length > 0) {
			    var ifrm = document.getElementById('downloadframe');
		    	ifrm.src = lmBase.url + 'f/streamZip/LimboDownloadFiles.zip?ids=' + lmBase.arrayToCSV(ids);
		    }
		};
		
		
		ctrl.getSelectedFiles = function(typefilter) {
			var result = new Array();
			
			for(x in ctrl.files) {
				var it = ctrl.files[x];
				if(it.client.select && (typefilter == null || typefilter(it.type))) {
					result.push(it);
				}
			}
			
			return result;
		};
		
		/* ============ */
		/* = FILE OPS = */
		/* ============ */	
		
		
		ctrl.fileEdit = function(valueItem) {
			var modalInstance = $modal.open({
			      templateUrl: 'dialogs/dialogFileEdit.html',
			      controller: 'CtrlDialogFileEdit',
			      resolve: {id: function () { return valueItem.id; }}
			    });
			 
			modalInstance.result.then(
				function () { ctrl.folderReload(); },
			    function () { /* dismissed */ }
			);
		};	
		
		ctrl.defaultFileAction = function(valueFile) {
			if('AUDIO' == valueFile.type || 'VIDEO' == valueFile.type) serviceMediaPlayer.playFile(valueFile);
			else if('IMAGE' == valueFile.type) ctrl.imageShow(valueFile);
			else ctrl.downloadFile(valueFile);
		};
		
		ctrl.downloadFile = function(item) {
			var ifrm = document.getElementById('downloadframe');
			ifrm.src = lmBase.url + 'f/stream/' + item.id + '/' + item.name;
		};
		
		ctrl.mediaPlay = function(valueItem) {
			serviceMediaPlayer.playFile(valueItem);
		};
		
		ctrl.mediaAdd = function(valueItem) {
			serviceMediaPlayer.addFile(valueItem);
		};
		
		ctrl.mediaPlayAll = function() {
			serviceMediaPlayer.playFiles(ctrl.getSelectedFiles(function(type) {return 'AUDIO' == type || 'VIDEO' == type}));
		};
		
		ctrl.mediaAddAll = function() {
			serviceMediaPlayer.addFiles(ctrl.getSelectedFiles(function(type) {return 'AUDIO' == type || 'VIDEO' == type}));
		};	
		
		ctrl.mediaExternal = function(valueItem) {
			var modalInstance = $modal.open({
			      templateUrl: 'dialogs/dialogExternalLinks.html',
			      controller: 'CtrlDialogExternalLinks',
			      resolve: {
			        video: function () {
			          return valueItem;
			        }
			      }
			    });
			 
			modalInstance.result.then(
				function () {
					
			    },
			    function () {
			    	// dismissed
		    });		
		};
		
		/* ============= */
		/* = NEW STUFF = */
		/* ============= */
		ctrl.uploadFiles = new Array();
		
		/* Functions for direct JS calls (non-angular) */	
		ctrl.onFileSelect = function(element) {
			$scope.$apply(function() {
				var files = new Array();
			
				for(x in element.files) {
					var el = element.files[x];
					if(el instanceof File) {
						files.push(el);
					}
				};
				/* let upload service manage this */
				var par = new Object();
				par.folder = ctrl.folder == null ? 0 : ctrl.folder.id;
				serviceUpload.uploadFiles(files,lmBase.url + 'f/file', false, par);
				
				// clear field
				element.innerHTML = element.innerHTML;
			});
		};
		
		/* init filedrop */
		var dropbox = document.getElementById('fileDrop');
		dropbox.ondragover = function(ev) {
			ev.preventDefault();
		};
		dropbox.ondrop = function(ev) {
			ev.stopPropagation();
			ev.preventDefault();
			var dt = ev.dataTransfer;
			if(dt.files.length > 0) {
				ctrl.onFileSelect(dt);
			}
		}
		serviceUpload.registerListener(ctrl.folderReload);
		
		
		/* destructor */
		$scope.$on('$destroy', function iVeBeenDismissed() {
			serviceUpload.registerListener(null);
		});	
		
		
		ctrl.folderCreate = new Object();
		ctrl.folderCreate.name = '';		/* new folders name */

		ctrl.folderCreate.create = function() {
			var req = {name: ctrl.folderCreate.name };
			req.parent = ctrl.folder == null ? 0 : ctrl.folder.id;
			
			$http.post(lmBase.url + 'f/folder',req)
				.success(function(data) {
					ctrl.folderReload();
					ctrl.folderCreate.name = '';
			}).error(serviceLimboMedia.handleHttpError);
		};
		

		/* ========= */
		/* = IMAGE = */
		/* ========= */
		ctrl.elImageFrame = document.getElementById('ImageFrame');
		ctrl.fs = serviceFullscreen.state;
		ctrl.imageCurrent = null;
		ctrl.imageIdx = 0;

		ctrl.imageShow = function(valueFile) {
			for(x in ctrl.files) {
				var currentItem = ctrl.files[x];
				if(currentItem.id == valueFile.id) {
					ctrl.imageIdx = parseInt(x);
					ctrl.imageCurrent = valueFile;
					serviceFullscreen.onToggleFullscreen(ctrl.elImageFrame);
					break;
				}
			}
		};
		
		ctrl.imageNext = function() {
			for (i = parseInt(ctrl.imageIdx) + 1; i < ctrl.files.length; i++) {
				var it = ctrl.files[i];
				if('IMAGE' == it.type) {
					ctrl.imageIdx = parseInt(i);
					ctrl.imageCurrent = it;
					return;
				}
			}
			serviceFullscreen.onToggleFullscreen(ctrl.elImageFrame);
		};

		ctrl.imagePrev = function() {
			for (i = parseInt(ctrl.imageIdx) - 1; i >= 0; i--) {
				var it = ctrl.files[i];
				if('IMAGE' == it.type) {
					ctrl.imageIdx = parseInt(i);
					ctrl.imageCurrent = it;
					return;
				}
			}
			serviceFullscreen.onToggleFullscreen(ctrl.elImageFrame);		
		};
		
		ctrl.onClickFullscreen = function(evt) {
			var third = screen.width / 3;
			if(evt.clientX < third) {
				ctrl.imagePrev();
			}
			else if(evt.clientX > (third * 2)) {
				ctrl.imageNext();
			}
			else {
				serviceFullscreen.onToggleFullscreen(ctrl.elImageFrame);
			}
		};	

		/* ============ */
		/* = SETTINGS = */
		/* ============ */
		ctrl.settings = new Object();
			
		ctrl.toggleConfig = function() {
			var modalInstance = $modal.open({
				templateUrl : 'dialogs/dialogFileSettings.html',
				controller : 'CtrlDialogFileSettings',
			});
			modalInstance.result.then(
				function () { ctrl.folderReload(); },
				function () { ctrl.folderReload(); }
			);
		};
		
		ctrl.saveSettings = function() {
			serviceLimboMedia.saveLocalSettings('files');
		};	
		
		ctrl.loadSettings = function() {
			ctrl.settings = serviceLimboMedia.getLocalSettings('files');

			if(ctrl.settings.view == null) ctrl.settings.view = 'LIST';
			if(ctrl.settings.pageSize == null) ctrl.settings.pageSize = 100;
			if(ctrl.settings.list == null) ctrl.settings.list = new Object();
			if(ctrl.settings.list.order == null) ctrl.settings.list.order = 'NAME';
			if(ctrl.settings.list.colType == null) ctrl.settings.list.colType = true;
			if(ctrl.settings.list.colSize == null) ctrl.settings.list.colSize = true;
			if(ctrl.settings.list.colAdded == null) ctrl.settings.list.colAdded = false;
			if(ctrl.settings.audio == null) ctrl.settings.audio = new Object();
			if(ctrl.settings.audio.order == null) ctrl.settings.audio.order = 'TITLE';
			if(ctrl.settings.audio.colAlbum == null) ctrl.settings.audio.colAlbum = false;
			if(ctrl.settings.audio.colGenre == null) ctrl.settings.audio.colGenre = false;
			if(ctrl.settings.audio.colTrack == null) ctrl.settings.audio.colTrack = false;
			if(ctrl.settings.audio.colYear == null) ctrl.settings.audio.colYear = false;
			if(ctrl.settings.audio.colDuration == null) ctrl.settings.audio.colDuration = true;
			if(ctrl.settings.audio.colSize == null) ctrl.settings.audio.colSize = false;
			if(ctrl.settings.gallery == null) ctrl.settings.gallery = new Object();
			if(ctrl.settings.gallery.order == null) ctrl.settings.gallery.order = 'NAME';
				
			ctrl.view = ctrl.settings.view;

			/* =========== */
			/* = STARTUP = */
			/* =========== */
			ctrl.folderLoad(ctrl.loadFolderID);
		}
		
		ctrl.$routerOnActivate = function(next) {
		    var id = next.params.id;
		    if(id) {
		    	ctrl.loadFolderID = id;
		    }
		    ctrl.loadSettings();
		};
	  
  }
});