lm.controller('CtrlDialogBookmarkEdit', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'serviceNotification', 'itemId', function($scope, $modalInstance, $http, serviceLimboMedia, serviceNotification, itemId) {
	$scope.itemId = itemId;
	$scope.item = null;
	$scope.labels = [];
	
	$scope.loadBookmark = function() {
		$http.get(lmBase.url + 'b/item/' + $scope.itemId)
		.success(function(data) {
			$scope.item = data;
		}).error(serviceLimboMedia.handleHttpError);	
	};
	
	$scope.loadLabels = function() {
		$http.get(lmBase.url + 'b/labels/writeable')
		.success(function(data) {
			$scope.labels = data;
		}).error(serviceLimboMedia.handleHttpError);	
	};
	
	$scope.loadBookmark();
	$scope.loadLabels();
	
	$scope.ok = function() {
		var req = {name: $scope.item.name, url: $scope.item.url, labelId: $scope.item.label.id};
		$http.post(lmBase.url + 'b/item/' + $scope.item.id, req)
			.success(function(data) {
				$modalInstance.close();
			}).error(serviceLimboMedia.handleHttpError);
	};
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
}]);

lm.controller('CtrlDialogBookmarkSettings', ['$scope', '$modalInstance', 'serviceLimboMedia', function($scope, $modalInstance, serviceLimboMedia) {
	$scope.settings = null;
	
	$scope.saveSettings = function() {
		serviceLimboMedia.saveLocalSettings('bookmarks');
	};
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
	
	$scope.settings = serviceLimboMedia.getLocalSettings('bookmarks');
	if($scope.settings.colName == null) $scope.settings.colName = true;
	if($scope.settings.colUrl == null) $scope.settings.colUrl = true;
	if($scope.settings.colAdded == null) $scope.settings.colAdded = false;
}]);


lm.controller('CtrlDialogExternalLinks', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'serviceNotification', 'video', function($scope, $modalInstance, $http, serviceLimboMedia, serviceNotification, video) {
	$scope.file = video;
	$scope.links = new Array();
	$scope.authtoken = '';
	$scope.transcodings = ['P360','P720','P1080'];
	
	$scope.removetranscoding = function(valuetrans) {
		var idx = $scope.transcodings.indexOf(valuetrans);
		if(idx != -1) $scope.transcodings.splice(idx, 1);
	};
	
	$scope.buildlinks = function() {
		var earl = lmBase.urlFull + lmBase.url;
		
		var link = new Object();
		link.name = 'LINK_ORIGINAL';
		link.link = earl + 'f/stream/' + $scope.file.id + '/' + encodeURIComponent($scope.file.name) + '?token=' + $scope.authtoken;
		$scope.links.push(link);
		
		if($scope.file.type == 'VIDEO') {
			for(x in $scope.file.formats) {
				var format = $scope.file.formats[x];
				var link = new Object();
				
				if(format.original) {
					link.name = 'Original';
					if(format.standard) { link.name += ', ' + format.standard; }
					if(format.layout) { link.name += ', ' + format.layout; }
					link.link = earl + 'f/stream/' + $scope.file.id + '/x?token=' + $scope.authtoken;
				}
				else {
					link.name = 'Converted, ' + format.standard + ', ' + format.layout;
					link.link = earl + 'f/streamVideo/' + $scope.file.id + '/x?token=' + $scope.authtoken + '&standard=' + format.standard + '&layout=' + format.layout;
				}
				
				if('WEBM' == format.standard) $scope.removetranscoding(format.layout);
				
				$scope.links.push(link);
			}
			
			for(x in $scope.transcodings) {
				var trans = $scope.transcodings[x];
				var link = new Object();
				link.name = 'Transcoding, WEBM, ' + trans;
				link.link = earl + 'f/streamVideo/' + $scope.file.id + '/x?token=' + $scope.authtoken + '&standard=WEBM&layout=' + trans;
				$scope.links.push(link);
			}
		}

	};
	
	$scope.aquireToken = function() {
		$http.get(lmBase.url + 'user/token/FILE/' + $scope.file.id)
		.success(function(data) {
			$scope.authtoken = data.id;
			$scope.buildlinks();
		})
		.error(serviceLimboMedia.handleHttpError);
	};
	
	$scope.aquireToken();
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
}]);

lm.controller('CtrlDialogFileEdit', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'serviceNotification', 'id', function($scope, $modalInstance, $http, serviceLimboMedia, serviceNotification, id) {
	$scope.id = id;			/* item id */
	$scope.item;
	$scope.vTranscodings = [
	                       {standard: 'WEBM',layout: 'P360',already: false},
	                       {standard: 'WEBM',layout: 'P720',already: false},
	                       {standard: 'WEBM',layout: 'P1080',already: false},
	                       {standard: 'MP4',layout: 'P360',already: false},
	                       {standard: 'MP4',layout: 'P720',already: false},
	                       {standard: 'MP4',layout: 'P1080',already: false}
	                       ];
	$scope.aTranscodings = [
	                       {standard: 'WEBM_VORBIS',already: false},
	                       {standard: 'MP3',already: false}
	                       ];
	
	$scope.load = function() {
		$http.get(lmBase.url + 'f/file/' + $scope.id)
			.success(function(data) {
				$scope.item = data;
				if('VIDEO' == $scope.item.type) {
					for(x in $scope.vTranscodings) {
						var vt = $scope.vTranscodings[x];
						for(y in $scope.item.formats) {
							var form = $scope.item.formats[y];
							if(vt.standard == form.standard && vt.layout == form.layout) {
								vt.already = true;
							}
						}
					}
				}
				if('AUDIO' == $scope.item.type) {
					for(x in $scope.aTranscodings) {
						var vt = $scope.aTranscodings[x];
						for(y in $scope.item.formats) {
							var form = $scope.item.formats[y];
							if(vt.standard == form.standard) {
								vt.already = true;
							}
						}
					}
				}
			}).error(serviceLimboMedia.handleHttpError);	
	};
	
	$scope.load();

	$scope.save = function() {
		var req = {name: $scope.item.name};
		$http.post(lmBase.url + 'f/file/' + $scope.id,req)
			.success(function(data) {
				$modalInstance.close();
			}).error(serviceLimboMedia.handleHttpError);
	};
	
	$scope.doReindex = function() {
		$http.post(lmBase.url + 'f/file/' + $scope.id + '/reindex')
		.success(function(data) {
			$modalInstance.close();
		}).error(serviceLimboMedia.handleHttpError);		
	};
	
	$scope.doVTranscode = function(vTrans) {
		$http.post(lmBase.url + 'f/video/' + $scope.id + '/encode',vTrans)
			.success(function(data) {
				serviceNotification.ok('MSG_JOB_STARTED');
			}).error(serviceLimboMedia.handleHttpError);
	};
	
	$scope.doATranscode = function(aTrans) {
		$http.post(lmBase.url + 'f/audio/' + $scope.id + '/encode',aTrans)
			.success(function(data) {
				serviceNotification.ok('MSG_JOB_STARTED');
			}).error(serviceLimboMedia.handleHttpError);
	};
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
}]);

lm.controller('CtrlDialogFileSettings', ['$scope', '$modalInstance', 'serviceLimboMedia', function($scope, $modalInstance, serviceLimboMedia) {
	$scope.settings = null;
	$scope.pageSizes = [10,20,50,100,200,500,1000,100000];
	
	$scope.saveSettings = function() {
		serviceLimboMedia.saveLocalSettings('files');
	};
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
	
	$scope.settings = serviceLimboMedia.getLocalSettings('files');
	if($scope.settings.view == null) $scope.settings.view = 'LIST';
	if($scope.settings.pageSize == null) $scope.settings.pageSize = 100;
	if($scope.settings.list == null) $scope.settings.list = new Object();
	if($scope.settings.list.order == null) $scope.settings.list.order = 'NAME';
	if($scope.settings.list.colType == null) $scope.settings.list.colType = true;
	if($scope.settings.list.colSize == null) $scope.settings.list.colSize = true;
	if($scope.settings.list.colAdded == null) $scope.settings.list.colAdded = false;
	if($scope.settings.audio == null) $scope.settings.audio = new Object();
	if($scope.settings.audio.order == null) $scope.settings.audio.order = 'TITLE';
	if($scope.settings.audio.colAlbum == null) $scope.settings.audio.colAlbum = false;
	if($scope.settings.audio.colGenre == null) $scope.settings.audio.colGenre = false;
	if($scope.settings.audio.colTrack == null) $scope.settings.audio.colTrack = false;
	if($scope.settings.audio.colYear == null) $scope.settings.audio.colYear = false;
	if($scope.settings.audio.colDuration == null) $scope.settings.audio.colDuration = true;
	if($scope.settings.audio.colSize == null) $scope.settings.audio.colSize = false;
	if($scope.settings.gallery == null) $scope.settings.gallery = new Object();
	if($scope.settings.gallery.order == null) $scope.settings.gallery.order = 'NAME';

	if($scope.settings.colName == null) $scope.settings.colName = true;
	if($scope.settings.colUrl == null) $scope.settings.colUrl = true;
	if($scope.settings.colAdded == null) $scope.settings.colAdded = false;
}]);

lm.controller('CtrlDialogFolderDelete', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'folder', function($scope, $modalInstance, $http, serviceLimboMedia, folder) {
	$scope.folder = folder;
	$scope.showBindInfo = folder.fspath != null && folder.fspath != '';
	$scope.working = false;
	
	$scope.deleteFolder = function() {
		$scope.working = true;
		
		$http.delete(lmBase.url + 'f/folder/' + $scope.folder.id)
			.success(function(data) {
				$modalInstance.close();
			}).error(function(data, status) {
				serviceLimboMedia.handleHttpError(data,status);
				$modalInstance.close();
		});
	};
	
	$scope.unbindFolder = function() {
		$scope.working = true;
		
		$http.post(lmBase.url + 'f/folder/' + $scope.folder.id + '/unbind')
			.success(function(data) {
				$modalInstance.close();
			}).error(function(data, status) {
				serviceLimboMedia.handleHttpError(data,status);
				$modalInstance.close();
		});
	};
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
}]);

lm.controller('CtrlDialogFolderMount', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'serviceNotification', function($scope, $modalInstance, $http, serviceLimboMedia, serviceNotification) {

	$scope.name = '';
	$scope.path = '';
	$scope.browseFolder = null;
	
	$scope.browse = function(valuePath) {
		if(valuePath != null && valuePath != '') {
			$scope.path = valuePath;
		}
		else {
			$scope.path = '';
		}
		$http.get(lmBase.url + 'system/browseFolder',{params : {path:valuePath}})
			.success(function(data) {
				$scope.browseFolder = data;
				$scope.name = data.name;
			}).error(serviceLimboMedia.handleHttpError);
	};
	
	$scope.save = function() {
		if($scope.path && $scope.path != '') {
			var req = {parent: 0, name: $scope.name, path: $scope.path };
	
			$http.post(lmBase.url + 'f/folder',req)
				.success(function(data) {
					serviceNotification.ok('MSG_JOB_STARTED');
					$modalInstance.close();
			}).error(serviceLimboMedia.handleHttpError);
		}
	};	
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};

	$scope.browse('');
}]);

lm.controller('CtrlDialogFolderTokens', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'id', function($scope, $modalInstance, $http, serviceLimboMedia, id) {
	$scope.id = id;
	$scope.items = [];
	
	$scope.availableRights = ['READ','WRITE'];
	$scope.selectedRight = 'READ';
	$scope.availableRanges = [{name: 'DAY',val: 86400000},{name:'WEEK',val: 604800000},{name: 'MONTH',val: 2592000000},{name:'YEAR',val: 31536000000}];
	$scope.selectedRange = $scope.availableRanges[1];
	
	$scope.load = function() {
		$http.get(lmBase.url + 'f/folder/' + $scope.id + '/tokens')
		.success(function(data) {
			$scope.items = data;
			$scope.buildLinks();
		}).error(serviceLimboMedia.handleHttpError);	
	};
	
	$scope.buildLinks = function() {
		var earl = lmBase.urlFull + '#/link/';
		
		for(x in $scope.items) {
			var it = $scope.items[x];
			it.link = earl + it.id + '/' + it.resourceType + '/' + it.resourceId;
		}
	};
	
	$scope.create = function() {
		var req = {right: $scope.selectedRight, expires: $scope.selectedRange.val};
		$http.post(lmBase.url + 'f/folder/' + $scope.id + '/token', req)
			.success(function(data) {
				$scope.load();
			}).error(serviceLimboMedia.handleHttpError);
	};
	
	$scope.remove = function(theitem) {
		$http.delete(lmBase.url + 'f/folder/' + $scope.id + '/token/' + theitem.id)
		.success(function(data) {
			$scope.load();
		});		
	};
	
	$scope.clipboard = function(theitem) {
		console.log("Copy to clipboard");
	};
	
	$scope.load();

	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
}]);

lm.controller('CtrlDialogHomeNewBox', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'serviceNotification', 'createBoxCallback', function($scope, $modalInstance, $http, serviceLimboMedia, serviceNotification, createBoxCallback) {
	$scope.createBoxCallback = createBoxCallback;
	
	$scope.bookmarkLabel = null;
	$scope.bookmarkLabels = [];
	$scope.todoLabel = null;
	$scope.todoLabels = [];
	$scope.noteLabel = null;
	$scope.noteLabels = [];
	$scope.note = null;
	$scope.notes = [];
	$scope.noteRows = 5;
	$scope.hideOwn = false;
	$scope.hideForeign = false;
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
	
	$scope.loadNotes = function() {
		$scope.notes.splice(0,$scope.notes.length);
		if($scope.noteLabel != null) {
			var getparams = new Object();
			getparams.lid = $scope.noteLabel.id;
			$http.get(lmBase.url + 'n/items/list',{params : getparams})			
				.success(function(data) {
					$scope.notes = data;
					if(data.length > 0) $scope.note = data[0];
				}).error(serviceLimboMedia.handleHttpError);
		}
	};
	
	/* load data for selection */
	$http.get(lmBase.url + 'b/labels/all')
		.success(function(data) {
			$scope.bookmarkLabels = data;
			if(data.length > 0) $scope.bookmarkLabel = data[0];
		}).error(serviceLimboMedia.handleHttpError);
	
	$http.get(lmBase.url + 't/labels/all')
		.success(function(data) {
			$scope.todoLabels = data;
			if(data.length > 0) $scope.todoLabel = data[0];
		}).error(serviceLimboMedia.handleHttpError);

	$http.get(lmBase.url + 'n/labels/all')
		.success(function(data) {
			$scope.noteLabels = data;
			if(data.length > 0) $scope.noteLabel = data[0];
			$scope.loadNotes();
		}).error(serviceLimboMedia.handleHttpError);
	
	/* create box */
	$scope.createNewsBox = function() {
		var box = new Object();
		box.type = 'newsBox';
		box.name = 'LimboMedia News';
		$scope.createBoxCallback(box);
	};

	$scope.createBookmarkBox = function() {
		if($scope.bookmarkLabel != null) {
			var box = new Object();
			box.type = 'bookmarkBox';
			box.name = $scope.bookmarkLabel.name;
			box.labelID = $scope.bookmarkLabel.id;
			$scope.createBoxCallback(box);
		}
	};
	
	$scope.createTodoBox = function() {
		if($scope.todoLabel != null) {
			var box = new Object();
			box.type = 'todoBox';
			box.name = $scope.todoLabel.name;
			box.labelID = $scope.todoLabel.id;
			$scope.createBoxCallback(box);
		}
	};
	
	$scope.createNoteBox = function() {
		if($scope.note != null) {
			var box = new Object();
			box.type = 'noteBox';
			box.name = $scope.note.name;
			box.noteID = $scope.note.id;
			box.rows = $scope.noteRows;
			$scope.createBoxCallback(box);
		}
	};
	
	$scope.createThingsBox = function() {
		var box = new Object();
		box.type = 'thingsBox';
		box.iname = 'IOT';
		box.hideOwn = $scope.hideOwn;
		box.hideForeign = $scope.hideForeign;
		$scope.createBoxCallback(box);
	};	
}]);

lm.controller('CtrlDialogLabelDelete', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'serviceNotification', 'label', 'prefix', function($scope, $modalInstance, $http, serviceLimboMedia, serviceNotification, label, prefix) {
	$scope.label = label;
	$scope.newname = label.name;
	$scope.prefix = prefix;

	$scope.ok = function() {
		$http.delete(lmBase.url + $scope.prefix + '/label/' + $scope.label.id)
		.success(function(data) {
			$modalInstance.close();
		}).error(serviceLimboMedia.handleHttpError);
	};
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
}]);

lm.controller('CtrlDialogLabelEdit', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'serviceNotification', 'label', 'url', function($scope, $modalInstance, $http, serviceLimboMedia, serviceNotification, label, url) {
	$scope.label = label;
	$scope.url = url;
	$scope.name = label.name;
	
	$scope.ok = function() {
		var req = {name: $scope.name};
		$http.post($scope.url, req)
			.success(function(data) {
				$modalInstance.close();
			}).error(serviceLimboMedia.handleHttpError);
	};
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
}]);

lm.controller('CtrlDialogNoteEdit', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'serviceNotification', 'itemId', function($scope, $modalInstance, $http, serviceLimboMedia, serviceNotification, itemId) {
	$scope.itemId = itemId;
	$scope.item = null;
	$scope.labels = [];
	
	$scope.loadBookmark = function() {
		$http.get(lmBase.url + 'n/item/' + $scope.itemId)
		.success(function(data) {
			$scope.item = data;
		}).error(serviceLimboMedia.handleHttpError);	
	};
	
	$scope.loadLabels = function() {
		$http.get(lmBase.url + 'n/labels/writeable')
		.success(function(data) {
			$scope.labels = data;
		}).error(serviceLimboMedia.handleHttpError);	
	};
	
	$scope.loadBookmark();
	$scope.loadLabels();
	
	$scope.ok = function() {
		var req = {name: $scope.item.name, content: $scope.item.content, labelId: $scope.item.label.id};
		$http.post(lmBase.url + 'n/item/' + $scope.item.id, req)
			.success(function(data) {
				$modalInstance.close();
			}).error(serviceLimboMedia.handleHttpError);
	};
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
}]);

lm.controller('CtrlDialogNoteSettings', ['$scope', '$modalInstance', 'serviceLimboMedia', function($scope, $modalInstance, serviceLimboMedia) {
	$scope.settings = null;
	
	$scope.saveSettings = function() {
		serviceLimboMedia.saveLocalSettings('notes');
	};
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};

	$scope.settings = serviceLimboMedia.getLocalSettings('notes');
	if($scope.settings.colName == null) $scope.settings.colName = true;
	if($scope.settings.colAdded == null) $scope.settings.colAdded = false;
}]);

lm.controller('CtrlDialogRename', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'serviceNotification', 'id', 'url', 'name', function($scope, $modalInstance, $http, serviceLimboMedia, serviceNotification, id, url, name) {
	$scope.name = name;		/* current name */
	$scope.newname = name;	/* new name */
	$scope.url = url;		/* API URL */
	$scope.id = id;			/* item id */

	$scope.save = function() {
		var req = {name: $scope.newname};
		$http.post($scope.url,req)
			.success(function(data) {
				$modalInstance.close();
			}).error(serviceLimboMedia.handleHttpError);
	};
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
}]);

lm.controller('CtrlDialogShare', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'serviceNotification', 'user', 'url', function($scope, $modalInstance, $http, serviceLimboMedia, serviceNotification, user, url) {
	$scope.user = user;		/* owner */
	$scope.shares = [];		/* shares */
	$scope.url = url;		/* API URL */
	
	$scope.availableRights = ['READ','WRITE'];
	$scope.selectedRight = 'READ';
	$scope.availableUsers = [];
	$scope.selectedUser = null;
	
	$scope.add = function() {
		if($scope.user.id == $scope.selectedUser.id) {
			serviceNotification.error('MSG_SHARE_YOURSELF');
			return;
		}
		
		for(x in $scope.shares) {
			if($scope.selectedUser.id == $scope.shares[x].user.id) {
				serviceNotification.error('MSG_SHARE_TWICE');
				return;
			}
		}
		
		var uwr = {user: $scope.selectedUser, right: $scope.selectedRight};
		$scope.shares.push(uwr);
	};
	
	$scope.remove = function(valueUserLabel) {
		for(x in $scope.shares) {
			if(valueUserLabel == $scope.shares[x]) {
				$scope.shares.splice(x,1);
				return;
			}
		}
	};	
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
	
	$scope.save = function() {
		$http.post($scope.url,$scope.shares)
			.success(function(data) { $modalInstance.close(); })
			.error(serviceLimboMedia.handleHttpError);
	};

	$scope.loadShares = function() {
		$http.get($scope.url)
		.success(function(data) {
			$scope.shares = data;
		}).error(serviceLimboMedia.handleHttpError);		
	};
	
	$scope.loadAvailableUsers = function() {
		$http.get(lmBase.url + 'user/users')
			.success(function(data) {
				$scope.availableUsers = data;
				$scope.selectedUser = $scope.availableUsers[0];
			}).error(serviceLimboMedia.handleHttpError);
	};
	
	$scope.loadAvailableUsers();
	$scope.loadShares();
}]);

lm.controller('CtrlDialogShareSimple', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'serviceNotification', 'user', 'url', function($scope, $modalInstance, $http, serviceLimboMedia, serviceNotification, user, url) {
	$scope.user = user;		/* owner */
	$scope.shares = [];		/* shares */
	$scope.url = url;		/* API URL */
	
	$scope.availableUsers = [];
	$scope.selectedUser = null;
	
	$scope.addUser = function() {
		if($scope.user.id == $scope.selectedUser.id) {
			serviceNotification.error('MSG_SHARE_YOURSELF');
			return;
		}
		
		for(x in $scope.shares) {
			if($scope.selectedUser.id == $scope.shares[x].id) {
				serviceNotification.error('MSG_SHARE_TWICE');
				return;
			}
		}

		$scope.shares.push($scope.selectedUser);
	};
	
	$scope.removeUser = function(valueUser) {
		for(x in $scope.shares) {
			if(valueUser == $scope.shares[x]) {
				$scope.shares.splice(x,1);
				return;
			}
		}
	};	
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};

	$scope.save = function() {
		$http.post($scope.url,$scope.shares)
			.success(function(data) { $modalInstance.close(); })
			.error(serviceLimboMedia.handleHttpError);
	};

	$scope.loadShares = function() {
		$http.get($scope.url)
		.success(function(data) {
			$scope.shares = data;
		}).error(serviceLimboMedia.handleHttpError);		
	};
	
	$scope.loadAvailableUsers = function() {
		$scope.availableUsers = new Array();
		$http.get(lmBase.url + 'user/users')
			.success(function(data) {
				$scope.availableUsers = data;
				$scope.selectedUser = $scope.availableUsers.length > 0 ? $scope.availableUsers[0] : null;
			}).error(serviceLimboMedia.handleHttpError);
	};
	
	$scope.loadAvailableUsers();
	$scope.loadShares();
}]);

lm.controller('CtrlDialogThingAction', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'serviceNotification', 'item', function($scope, $modalInstance, $http, serviceLimboMedia, serviceNotification, item) {
	$scope.actionTypes = ['','HTTP_GET','HTTP_POST','UDP'];
	$scope.item = item;
	$scope.action = {type: '', url: '', headers: [], host: '', port: 12345, content: ''};
		
	$scope.ok = function() {
		var req = new Object();
		if($scope.action.type != null && $scope.action.type != '') {
			req.action = $scope.action;
		}
		$http.post(lmBase.url + 'iot/item/' + $scope.item.id + '/action',req)
			.success(function(data) { $modalInstance.close(); })
			.error(serviceLimboMedia.handleHttpError);
	};
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
	
	$scope.loadAction = function() {
		$http.get(lmBase.url + 'iot/item/' + $scope.item.id + '/action')
		.success(function(data) {
			if(data.action != null) {
				$scope.action = data.action;
			}
		}).error(serviceLimboMedia.handleHttpError);
	};
	$scope.loadAction();
}]);

lm.controller('CtrlDialogThingCrup', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'serviceNotification', 'item', function($scope, $modalInstance, $http, serviceLimboMedia, serviceNotification, item) {
	$scope.sensorTypes = ["SENSOR_BOOLEAN", "SENSOR_COORDINATE", "SENSOR_FIRE", "SENSOR_NUMBER", "SENSOR_STRING", "CONTROL_SWITCH", "CONTROL_FIRE", "CONTROL_SELECT", "CONTROL_PERCENT", "CONTROL_TEXT", "CONTROL_NUMBER"];
	$scope.item = item;
	
	$scope.loadUid = function() {
		$http.get(lmBase.url + 'util/uid')
		.success(function(data) {
			$scope.req.uid = data;
		}).error(serviceLimboMedia.handleHttpError);
	}

	if(item == null) {
		$scope.create = true;
		$scope.req = {name: '', uid: '', type: '', unit: '', persistent: true, type: $scope.sensorTypes[0], selectStringOptions: []};
		$scope.loadUid();
	}
	else {
		$scope.create = false;
		$scope.req = {name: item.name, uid: item.uid, persistent: item.persistent, type: item.type, unit: item.unit, type: item.type, selectStringOptions: angular.copy(item.options)};
	}
	
	$scope.doShrink = function(valMode,valTimeunit) {
		var re = {mode: valMode, timeUnit: valTimeunit};
		$http.post(lmBase.url + 'iot/item/' + $scope.item.id + '/aggregate', re)
		.success(function(data) {
			$modalInstance.close();
		}).error(serviceLimboMedia.handleHttpError);
	};
	
	$scope.ok = function() {
		if($scope.create) {
			$http.post(lmBase.url + 'iot/item', $scope.req)
				.success(function(data) {
					$modalInstance.close();
				}).error(serviceLimboMedia.handleHttpError);
		}
		else {
			$http.post(lmBase.url + 'iot/item/' + $scope.item.id,$scope.req)
			.success(function(data) {
				$modalInstance.close();
			}).error(serviceLimboMedia.handleHttpError);
		}
	};
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
}]);

lm.controller('CtrlDialogThingDelete', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'serviceNotification', 'sensor', function($scope, $modalInstance, $http, serviceLimboMedia, serviceNotification, sensor) {
	$scope.sensor = sensor;

	$scope.ok = function() {
		$http.delete(lmBase.url + 'iot/item/' + $scope.sensor.id)
			.success(function(data) {
				$modalInstance.close();
			})
			.error(serviceLimboMedia.handleHttpError);
	};
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
}]);

lm.controller('CtrlDialogThingPull', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'serviceNotification', 'item', function($scope, $modalInstance, $http, serviceLimboMedia, serviceNotification, item) {
	$scope.actionTypes = ['','HTTP_GET','HTTP_POST'];
	$scope.item = item;
	$scope.action = {type: '', url: '', headers: []};
	$scope.cron = '';
	
	$scope.ok = function() {
		var req = {};
		if($scope.cron != '') req.cron = $scope.cron;
		if($scope.action.type != null && $scope.action.type != '') {
			req.action = $scope.action;
		}		
		$http.post(lmBase.url + 'iot/item/' + $scope.item.id + '/pull',req)
			.success(function(data) { $modalInstance.close(); })
			.error(serviceLimboMedia.handleHttpError);
	};
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
	
	$scope.loadAction = function() {
		$http.get(lmBase.url + 'iot/item/' + $scope.item.id + '/pull')
		.success(function(data) {
			$scope.cron = data.cron;
			if(data.action != null) {
				$scope.action = data.action;
			}
		}).error(serviceLimboMedia.handleHttpError);		
	};
	$scope.loadAction();
}]);

lm.controller('CtrlDialogThingPush', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'serviceNotification', 'sensor', function($scope, $modalInstance, $http, serviceLimboMedia, serviceNotification, sensor) {
	$scope.sensor = sensor;
	$scope.url = lmBase.urlFull + lmBase.url + 'iot/push/' + sensor.uid;
	
	var body = '{"ts": 1447284448000 {0}}';
	
	if('SENSOR_NUMBER' == sensor.type) {
		body = body.replace('{0}',', "val": 21.5');	
	}
	if('SENSOR_STRING' == sensor.type) {
		body = body.replace('{0}',', "val": "Some value"');	
	}
	if('SENSOR_BOOLEAN' == sensor.type) {
		body = body.replace('{0}',', "val": true');	
	}
	if('SENSOR_FIRE' == sensor.type) {
		body = body.replace('{0}','');
	}
	if('SENSOR_COORDINATE' == sensor.type) {
		body = body.replace('{0}',', "x": 11.5754900, "y": 48.1374300');	
	}
	
	$scope.body = body;
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
}]);

lm.controller('CtrlDialogTodoEdit', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'serviceNotification', 'itemId', function($scope, $modalInstance, $http, serviceLimboMedia, serviceNotification, itemId) {
	$scope.itemId = itemId;
	$scope.item = null;
	$scope.labels = [];
	
	$scope.loadBookmark = function() {
		$http.get(lmBase.url + 't/item/' + $scope.itemId)
		.success(function(data) {
			$scope.item = data;
		}).error(serviceLimboMedia.handleHttpError);	
	};
	
	$scope.loadLabels = function() {
		$http.get(lmBase.url + 't/labels/writeable')
		.success(function(data) {
			$scope.labels = data;
		}).error(serviceLimboMedia.handleHttpError);	
	};
	
	$scope.loadBookmark();
	$scope.loadLabels();
	
	$scope.ok = function() {
		var req = {name: $scope.item.name, done: $scope.item.done, labelId: $scope.item.label.id};
		$http.post(lmBase.url + 't/item/' + $scope.item.id, req)
			.success(function(data) {
				$modalInstance.close();
			}).error(serviceLimboMedia.handleHttpError);
	};
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
}]);

lm.controller('CtrlDialogTodoSettings', ['$scope', '$modalInstance', 'serviceLimboMedia', function($scope, $modalInstance, serviceLimboMedia) {
	$scope.settings = null;
	
	$scope.saveSettings = function() {
		serviceLimboMedia.saveLocalSettings('todos');
	};
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};

	$scope.settings = serviceLimboMedia.getLocalSettings('todos');
}]);

lm.controller('CtrlDialogUserCreate', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'serviceNotification', function($scope, $modalInstance, $http, serviceLimboMedia, serviceNotification) {
	$scope.availableAuthorities = lmBase.authorities;
	$scope.availableLanguages = new Array();
	
	$scope.item = {name: '', password: '', language: null, authorities: []};
	lmBase.multiSelectionInit($scope.item.authorities,$scope.availableAuthorities,['ROLE_WRITE']);

	$scope.ok = function() {
		var req = {name: $scope.item.name, password: $scope.item.password, language: $scope.item.language.code}
		req.authorities = lmBase.multiSelectionGetSelected($scope.item.authorities);
		
		$http.post(lmBase.url + 'userAdmin/user',req)
		.success(function(data) {
			$modalInstance.close();
		}).error(serviceLimboMedia.handleHttpError);		
	};
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
	
	$scope.loadLanguages = function() {
		$http.get(lmBase.url + 'system/info')
			.success(function(data) {
				$scope.availableLanguages = data.languages;
				$scope.item.language = data.languages[0];
			}).error(serviceLimboMedia.handleHttpError);	
	};
	
	$scope.loadLanguages();	
}]);

lm.controller('CtrlDialogUserDelete', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'serviceNotification', 'item', function($scope, $modalInstance, $http, serviceLimboMedia, serviceNotification, item) {
	$scope.item = item;

	$scope.ok = function() {
		$http.delete(lmBase.url + 'userAdmin/user/' + $scope.item.id)
			.success(function(data) {
				serviceNotification.ok('MSG_JOB_STARTED');
				$modalInstance.close();
			}).error(serviceLimboMedia.handleHttpError);
	};
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
}]);


lm.controller('CtrlDialogUserEdit', ['$scope', '$modalInstance', '$http', 'serviceLimboMedia', 'serviceNotification', 'item', function($scope, $modalInstance, $http, serviceLimboMedia, serviceNotification, item) {
	$scope.availableAuthorities = lmBase.authorities;
	$scope.availableLanguages = new Array();
	
	$scope.item = item;
	$scope.authorities = [];
	lmBase.multiSelectionInit($scope.authorities,$scope.availableAuthorities,$scope.item.authorities);

	$scope.ok = function() {
		var req = {name: $scope.item.name, password: $scope.item.password, language: $scope.item.language.code, description: $scope.item.description, enable: $scope.item.enable}
		req.authorities = lmBase.multiSelectionGetSelected($scope.authorities);
		
		$http.post(lmBase.url + 'userAdmin/user/' + $scope.item.id,req)
		.success(function(data) {
			$modalInstance.close();
		}).error(serviceLimboMedia.handleHttpError);		
	};
	
	$scope.cancel = function() {
		$modalInstance.dismiss('cancel');
	};
	
	$scope.loadLanguages = function() {
		$http.get(lmBase.url + 'system/info')
			.success(function(data) {
				$scope.availableLanguages = data.languages;
				$scope.item.language = data.languages[0];
			}).error(serviceLimboMedia.handleHttpError);	
	};
	
	$scope.loadLanguages();	
}]);
