/* ===================================== */
/* = Global vars and utility functions = */
/* ===================================== */

var lmBase = new Object();
lmBase.urlFull = document.getElementById("pixel").src.replace("img/pixel.png","");
lmBase.urlSock = 'ws' + lmBase.urlFull.substr(4);
lmBase.url = 'api/';

lmBase.authorities = ['ROLE_EDIT_USER_DETAILS','ROLE_WRITE','ROLE_ACCESS_FILESYSTEM','ROLE_NETWORK','ROLE_ADMINISTRATION'];

lmBase.niceSize = function (bytes) {
	if (bytes == null) return '';
    if (bytes == 0) return 'n/a';
    var sizes = ['b', 'kb', 'mb', 'gb', 'tb', 'pb', 'eb', 'zb', 'yb'],
    i = parseInt(Math.floor(Math.log(bytes) / Math.log(1024)));
    return Math.round(bytes / Math.pow(1024, i), 2) + ' ' + sizes[i];
};

lmBase.secondsToString = function(valueSeconds) {
	if(isFinite(String(valueSeconds))) {
		var dur = moment.duration(valueSeconds * 1000);
		var ss = dur.seconds();
		if(ss < 10) ss = '0' + ss;
		var mm = dur.minutes();
		if(mm < 10) mm = '0' + mm;
		var hh = dur.hours();
		if(hh < 10) hh = '0' + hh;
		return hh + ':' + mm + ':' + ss;
	}
	else return 'unknown';
};

lmBase.timestampToDate = function(valueTimestamp) {
	var x = moment(valueTimestamp);
	if(x.isValid()) {
		return x.format("YYYY-MM-DD");
	}
	else {
		return 'unknown';
	}
};

lmBase.timestampToDateTime = function(valueTimestamp) {
	var x = moment(valueTimestamp);
	if(x.isValid()) {
		return x.format("YYYY-MM-DD HH:mm:ss");
	}
	else {
		return 'unknown';
	}
};

lmBase.getOffset = function(el) {
    var _x = 0;
    var _y = 0;
    var _w = el.offsetWidth;
    while( el && !isNaN( el.offsetLeft ) && !isNaN( el.offsetTop ) ) {
        _x += el.offsetLeft - el.scrollLeft;
        _y += el.offsetTop - el.scrollTop;
        el = el.offsetParent;
    }
    return { top: _y, left: _x, width: _w};
};

lmBase.cutToAscii = function(val) {
	return val.replace(/[^\x00-\x7F]/g, "");
};

lmBase.cutToLatin1 = function(val) {
	return val.replace(/[^\x00-\xFF]/g, "");
};

lmBase.mouseInElement = function(evt) {
	return lmBase.mouseInDiv(evt.currentTarget,evt);
}

lmBase.mouseInDiv = function(el,evt) {
	var xmouse = evt.clientX;
	var divCords = lmBase.getOffset(el);
	var res = (xmouse - divCords.left) / divCords.width;
	return res; 
};

lmBase.arrayToCSV = function(values) {
	var csvstring = '';
		for(x in values) {
			var val = values[x];
			if(csvstring != '') csvstring += ',';
			csvstring += val.id;
		}
	return csvstring;
};

lmBase.arrayToIdArray = function(values) {
	var result = [];
	for(x in values) {
		var val = values[x];
		result.push(val.id);
	}
	return result;
};


lmBase.stringArrayToCSV = function(values) {
	var csvstring = '';
		for(x in values) {
			var val = values[x];
			if(csvstring != '') csvstring += ',';
			csvstring += val;
		}
	
	return csvstring;
};

lmBase.multiSelectionInit = function(targetArray, items, selected) {
	targetArray.splice(0,targetArray.length);
	for(x in items) {
		var ok = new Object();
		ok.selected = false;
		if(selected != null) {
			for(y in selected) {
				if(selected[y] == items[x]) {
					ok.selected = true;
				}
			};
		};
		ok.item = items[x];
		targetArray.push(ok);
	}
};

lmBase.multiSelectionGetSelected = function(valueArray) {
	var result = new Array();
	for(x in valueArray) {
		var item = valueArray[x];
		if(item.selected) {
			result.push(item.item);
		}
	}
	return result;
};

lmBase.arrayRemoveById = function(valueArray, idToRemove) {
	for(x in valueArray) {
		var it = valueArray[x];
		if(it.id == idToRemove) {
			valueArray.splice(x,1);
			break;
		}
	}
};

/* filter takes item and returns boolean */
lmBase.arrayRemoveByFilter = function(valueArray, filter) {
	for(x in valueArray) {
		var it = valueArray[x];
		if(filter(it)) {
			valueArray.splice(x,1);
			break;
		}
	}	
};

lmBase.isFullscreen = function() {
	return (document.fullscreenElement || document.webkitFullscreenElement || document.mozFullScreenElement || document.msFullscreenElement) ? true : false;
};

lmBase.toggleFullscreen = function(enable, element) {
	if(enable) {
		if(element.requestFullscreen) element.requestFullscreen();
		else if(element.webkitRequestFullscreen) element.webkitRequestFullscreen();
		else if(element.mozRequestFullScreen) element.mozRequestFullScreen();
		else if(element.msRequestFullscreen) element.msRequestFullscreen();				
	}
	else {
		if (document.exitFullscreen) document.exitFullscreen();
		else if (document.webkitExitFullscreen) document.webkitExitFullscreen();
		else if (document.mozCancelFullScreen) document.mozCancelFullScreen();
		else if (document.msExitFullscreen) document.msExitFullscreen();
	}
};

lmBase.pimpLmFile = function(valueItem) {
	valueItem.client = new Object();
	valueItem.client.select = false;
	valueItem.client.downurl = lmBase.url + 'f/stream/' + valueItem.id + '/' + encodeURIComponent(valueItem.name);
	
	if(valueItem.type == 'IMAGE') {
		valueItem.client.urlThumbS = lmBase.url + 'f/streamImage/' + valueItem.id + '/THUMB_S/' + valueItem.name;
		valueItem.client.urlThumbL = lmBase.url + 'f/streamImage/' + valueItem.id + '/THUMB_L/' + valueItem.name;
		valueItem.client.urlM = lmBase.url + 'f/streamImage/' + valueItem.id + '/M/' + valueItem.name;
		valueItem.client.dateCapture = (valueItem.dateCapture == 0 ? "" : moment(valueItem.dateCapture).format('YYYY-MM-DD hh:mm:ss'));
	}
};

