lm.directive('lmRole', function(serviceLimboMedia, $animate) {
	return {
	    restrict: 'A',
	    multiElement: true,
	    link: function(scope, element, attr) {
	    	scope.userdetails = serviceLimboMedia.userdetails;
	    	scope.$watch('userdetails.user', function(value) {
				var hasit = serviceLimboMedia.hasRole(attr.lmRole);
				$animate[hasit ? 'removeClass' : 'addClass'](element, 'lmHide', { });
				if(!hasit) $animate['removeClass'](element, 'lmFloatBox', { });
	    	});
	    }
	  };
});

lm.directive('lmShowIn', function(serviceLimboMedia, $animate) {
	return {
	    restrict: 'A',
	    multiElement: true,
	    link: function(scope, element, attr) {
	    	scope.userdetails = serviceLimboMedia.userdetails;
	    	scope.$watch('userdetails.user', function(value) {
				$animate[value ? 'removeClass' : 'addClass'](element, 'lmHide', { });
	    	});
	    }
	  };
});

lm.directive('lmHideIn', function(serviceLimboMedia, $animate) {
	return {
	    restrict: 'A',
	    multiElement: true,
	    link: function(scope, element, attr) {
	    	scope.userdetails = serviceLimboMedia.userdetails;
	    	scope.$watch('userdetails.user', function(value) {
				$animate[value ? 'addClass' : 'removeClass'](element, 'lmHide', { });
	    	});
	    }
	  };
});

lm.directive('lmSensorgraph', function($http, serviceNotification) {
	
	function link(scope, element, attrs) {
		var sensordata;
	
		scope.$watch(attrs.lmSensorgraph, function(value) {
			sensordata = value;
			scope.drawFile();
      	});

		scope.drawFile = function() {
			if(sensordata == null) return;
			
			var sd = new Array();
			sd.push(sensordata);
			
			var graph = $.plot($(element), sd, {
				legend: {show:true, position:"ne",noColumns:1},
				lines: {show:true},
				xaxis: {
					mode: "time",
					timezone: "browser",
					timeformat: "%Y-%m-%d<br/>%H:%M:%S"
				},
				selection: { mode: "x" },
				tooltip: true,
				tooltipOpts: { content: "<h4>%s</h4> Value: %y.3<br/>%x" },
				grid: { hoverable: true }				
			});
		};
	};
    return { link: link };
});

lm.directive('lmGraph', function(serviceNotification) {
	
	function link(scope, element, attrs) {
		var file;
		
		scope.$watch(attrs.lmRrd, function(value) {
			scope.drawFile(file);
		});
	
		scope.$watch(attrs.lmGraph, function(value) {
			file = value;
			if(value.rrdFile) {
				scope.drawFile(value);
			}
			else {
				var earl = lmBase.url + 'rrd/file/' + value.id; 
				FetchBinaryURLAsync(earl,function(bf, valueFile) {scope.$apply(scope.onRrdFileLoaded(bf,valueFile));},value);
			}
      	});

		scope.onRrdFileLoaded = function(bf,valueFile) {
			if (bf.getLength()<1) {
				// load error or file empty
	    		return;
	  		}
	  		
			valueFile.rrds = new Array();
			valueFile.rrdSelected = null;
			valueFile.dss = new Array();
			
			try {
				valueFile.rrdFile = new RRDFile(bf,new Object());
				
				var nrRRAs=valueFile.rrdFile.getNrRRAs();
				for (var i=0; i<nrRRAs; i++) {
					var rra=valueFile.rrdFile.getRRAInfo(i);
					
					var obj = new Object();
					obj.id = i;
					obj.step = rra.getStep();
					obj.rows = rra.getNrRows();
				    obj.period = obj.step*obj.rows; 
				    obj.name = rfs_format_time(obj.step)+" ("+rfs_format_time(obj.period)+" total)";
				    valueFile.rrds.push(obj);
				    if(i == 0) {
				    	valueFile.rrdSelected = obj;
				    }
				}
				
				var nrDSs=valueFile.rrdFile.getNrDSs();
				for (var i=0; i<nrDSs; i++) {
					var ds=valueFile.rrdFile.getDS(i);
					
					var obj = new Object();
					obj.id = i;
					obj.name = ds.getName();
					obj.type = ds.getType();
					valueFile.dss.push(obj);
				}
			} catch(err) {
				serviceNotification.error('MSG_RRD');
			}
		};
		
		scope.drawFile = function(valueFile) {
			if(!valueFile || !valueFile.rrdSelected) {
				return;
			}
		
			var ds_positive_stack_list=[];
			var ds_negative_stack_list=[];
			var ds_single_list=[];
			
			for(x in valueFile.dss) {
				ds_single_list.push(valueFile.dss[x].name);
			}
			
			// then extract RRA data about those DSs
  			var flot_obj=rrdRRAStackFlotObj(valueFile.rrdFile,valueFile.rrdSelected.id,ds_positive_stack_list,ds_negative_stack_list,ds_single_list,0*3600);			
			
			var graph_options = scope.graphOptions(valueFile);
						
			graph_options.xaxis.min=flot_obj.min;
			graph_options.xaxis.max=flot_obj.max;
    	
			var target = "#graph" + valueFile.id;
			var graph = $.plot($(target), flot_obj.data, graph_options);
		};
		
		scope.graphOptions = function(valueFile) {
			var result = {
				legend: {show:true, position:"nw",noColumns:3},
				lines: {show:true},
				xaxis: { mode: "time" },
				yaxis: { autoscaleMargin: 0.20},
				selection: { mode: "x" },
				tooltip: true,
				tooltipOpts: { content: "<h4>%s</h4> Value: %y.3" },
				grid: { hoverable: true },
			};
			
			if(valueFile.plugin == "uptime") {
				result.yaxis.tickFormatter = scope.tfUptime;
			}
			else if(valueFile.plugin == "cpufreq") {
				result.yaxis.tickFormatter = scope.tfCpuFreq;
			}
			else if(valueFile.plugin == "df") {
				result.yaxis.tickFormatter = scope.tfDf;
			}		
			else if(valueFile.name.indexOf("octets") != -1) {
				result.yaxis.tickFormatter = scope.tfIfOctets;
			}
			
			return result;
		};
		
		scope.tfUptime = function(val, axis) {
			if(val > 86400)
				return (val / 86400).toFixed(axis.tickDecimals) + " days";
			else if (val > 3600)
				return (val / 3600).toFixed(axis.tickDecimals) + " hrs";
			else if (val > 60)
				return (val / 60).toFixed(axis.tickDecimals) + " min";
			else
				return val.toFixed(axis.tickDecimals) + " secs";
		};		
		scope.tfDf = function(val, axis) {
			if (val > 10000000000)
				return (val / 1000000000).toFixed(axis.tickDecimals) + " GB";
			else if (val > 10000000)
				return (val / 1000000).toFixed(axis.tickDecimals) + " MB";
			else if (val > 1000)
				return (val / 1000).toFixed(axis.tickDecimals) + " kB";
			else
				return val.toFixed(axis.tickDecimals) + " B";
		};
		scope.tfIfOctets = function(val, axis) {
			if (val > 10000000)
				return (val / 1000000).toFixed(axis.tickDecimals) + " MB";
			else if (val > 1000)
				return (val / 1000).toFixed(axis.tickDecimals) + " kB";
			else
				return val.toFixed(axis.tickDecimals) + " B";
		};
		scope.tfCpuFreq = function(val, axis) {
			if (val > 1000000)
				return (val / 1000000).toFixed(axis.tickDecimals) + " mhz";
			else if (val > 1000)
				return (val / 1000).toFixed(axis.tickDecimals) + " khz";
			else
				return val.toFixed(axis.tickDecimals) + " mhz";
		};
	};
	
    return {
      link: link
    };
});

