lm.factory('serviceLimboMedia', [ '$q', '$http', '$location', '$rootScope', 'serviceNotification' , '$translate', '$rootRouter', function($q, $http, $location, $rootScope, serviceNotification, $translate, $rootRouter) {
	var service = new Object();
	
	/* consts */
	service.remoteSettingsName = 'webgui';
	service.initPages = [];
	service.initPages['HOME'] = 'Home';
	service.initPages['FILEMEDIA'] = 'Files';
	service.initPages['BOOKMARKS'] = 'Bookmarks';
	service.initPages['TODOS'] = 'Todos';
	service.initPages['NOTES'] = 'Notes';
	service.initPages['IOT'] = 'IoT';
	
	/* attributes */
	service.userdetails = {
		user: null,
		settingsRemote: null	/* remote storage settings */
	};

	service.settingsLocal = {};	/* local storage settings  */

	/* Promise answering isAuthenticated requests. Possible states: */
	/* null							--> on initialization and after logout */
	/* "return-true-promise"		--> after successfull login */
	/* "get-userdetails-promise"	--> no login but page requested auth */
	service.authpromise = null;

	try { service.haslocalstorage = 'localStorage' in window && window['localStorage'] !== null; }
	catch (e) { service.haslocalstorage = false; }

	service.getUserdetails = function() {
		return service.userdetails;
	}
	
	service.resetUser = function() {
		service.authpromise = null;
		service.userdetails.user = null;
		service.userdetails.settingsRemote = null;
		$translate.use('en');
		$rootScope.$emit('lmLogout');
	};

	service.setUser = function(userData) {
		service.resetUser();
		service.userdetails.user = userData;
	
		$translate.use(userData.language);
		
		try {
			service.userdetails.settingsRemote = angular.fromJson(userData.settings);
		} catch(e) {
			console.log("Error parsing user settings: " + e);
		}
		if(service.userdetails.settingsRemote == null || service.userdetails.settingsRemote == '') {
			service.userdetails.settingsRemote = {};
		}
		
		/* when user is set correctly, we let the authpromise always return true */
		service.authpromise = $q.resolve(true);
		
		$rootScope.$emit('lmLogin');
	};
	
	service.changeLang = function(lang) {
		$translate.use(lang);
		service.userdetails.user.language = lang;
	};
	
	service.handleHttpError = function(data, status) {
		console.log("Data: " + data);
		console.log("Status: " + status);
		if(401 == status) {
			service.resetUser();
			$rootRouter.navigate(['Login']);
		}
		else if(-1 == status) {
			serviceNotification.handleLimboError({key: 'CONNECTION_ERROR'});
		}
		else if(data && data.key) {
			serviceNotification.handleLimboError(data);
		}
		else {
			serviceNotification.error("Server Error:" + status);
		}
	};

	service.hasRole = function(valueRole) {
		if(service.userdetails.user == null) return false;
		return service.userdetails.user.authorities.indexOf(valueRole) >= 0;
	}
	
	
	service.isAuthenticated = function() {
		if(!service.authpromise) {
			service.authpromise = $q(function(resolve, reject) {
				$http.get(lmBase.url + 'user/user?settingKey=' + service.remoteSettingsName)
					.success(function(data) {
						service.setUser(data);
						resolve(true);
					})
					.error(function() {
						$rootRouter.navigate(['Login']);
						resolve(false);
					});
			});
		}
		return service.authpromise;
	}
	
	/* Get remote settings of current user. Requires isAuthenticated() to guarantee contents. */
	service.getRemoteSettings = function() {
		return service.userdetails.settingsRemote;
	}
	
	service.saveRemoteSettings = function() {
		var jsonToSave =  angular.toJson(service.userdetails.settingsRemote);
		$http.post(lmBase.url + 'setting/setting/' + service.remoteSettingsName, jsonToSave, httpCfgText)
			.success(function(data) { })
			.error(function(data, status) {
				if(403 != status) service.handleHttpError(data,status);
			});	
	};
	
	service.getLocalSettings = function(key) {
		if(service.settingsLocal[key] == null) {
			if(service.haslocalstorage) {
				var data = localStorage.getItem(key);
				service.settingsLocal[key] = (data == null || data == '') ? {} : angular.fromJson(data);  
			}
			else service.settingsLocal[key] = {};
		}
		return service.settingsLocal[key];
	};

	service.saveLocalSettings = function(key) {
		if(service.haslocalstorage) {
			var objectToSave = service.settingsLocal[key];
			var jsonToSave =  angular.toJson(objectToSave);
			localStorage.setItem(key, jsonToSave);
		}
	};
	
	/* external login logout functions */
	service.logout = function() {
		$http.post(lmBase.url + 'user/logout')
			.success(function(data) {
				service.resetUser();
				$rootRouter.navigate(['Login']);
			}).error(service.handleHttpError);
	};
	
	service.login = function(valueUser, valuePass) {
		if(valueUser == null || valuePass == null || valueUser.length == 0 || valuePass.length == 0) {
			serviceNotification.error('MSG_USER_PASS');
			return;
		}
		
		var req = {username: valueUser, password: valuePass, settingKey: service.remoteSettingsName};

		$http.post(lmBase.url + 'user/login',req)
			.success(function(data) {
				service.setUser(data);
				
				/* load initial page */
				var initPage = service.initPages[service.userdetails.settingsRemote.initialPage];
				if(!initPage) {
					initPage = service.initPages['HOME'];
				}
				$rootRouter.navigate([initPage]);
			}).error(service.handleHttpError);
	};

	return service;
}]);
