/* ========================== */
/* = SERVICE - Media player = */
/* ========================== */
lm.factory('serviceMediaPlayer', ['$rootScope', '$http', 'serviceNotification', 'serviceLimboMedia', 'serviceFullscreen', 'serviceSidebar', function($rootScope, $http, serviceNotification, serviceLimboMedia, serviceFullscreen, serviceSidebar) { 
	
	var service = new Object();
	
	/* some statics */
	service.transcodingFormats = [
 		{standard: 'WEBM',layout: 'P360',original: false,transcoding: true,mime: 'video/webm'},
 		{standard: 'WEBM',layout: 'P720',original: false,transcoding: true,mime: 'video/webm'},
 		{standard: 'WEBM',layout: 'P1080',original: false,transcoding: true,mime: 'video/webm'}
 	];
	service.layoutMap = new Object();
	service.layoutMap['P360'] = '360p ';
	service.layoutMap['P720'] = '720p ';
	service.layoutMap['P1080'] = '1080p';
	
	/* elements */
	service.elAudioPlayer = document.getElementById('AudioPlayer');
	service.elAudioBarPos = document.getElementById('AudioBarPos');
	service.elAudioBarVol = document.getElementById('AudioBarVol');
	service.elVideoPlayer = document.getElementById('VideoPlayer');
	service.elVideoFrame = document.getElementById('VideoFrame');
	service.elVideoControl = document.getElementById('VideoControl');
	service.elVideoBarPos = document.getElementById('VideoBarPos');
	service.elVideoBarVol = document.getElementById('VideoBarVol');
	service.elVideoScrollPoint = document.getElementById('VideoScrollPoint');
	service.elVideoPlayer._captionatorOptions = new Object();
	service.elVideoPlayer._captionatorOptions.appendCueCanvasTo = service.elVideoFrame;
	trackObject = null;
	
	/* detect supported formats */
	service.supportedAudio = new Array();	
	if(service.elAudioPlayer.canPlayType("audio/webm").match(/maybe|probably/i)) service.supportedAudio.push("WEBM_VORBIS");
	if(service.elAudioPlayer.canPlayType("audio/mp3").match(/maybe|probably/i)) service.supportedAudio.push("MP3");
	if(service.elAudioPlayer.canPlayType("audio/ogg").match(/maybe|probably/i)) service.supportedAudio.push("OGG_VORBIS");
	service.supportedVideo = new Array();
	service.supportedWEBM = false;
	service.supportedMP4 = false;
	if(service.elVideoPlayer.canPlayType("video/webm").match(/maybe|probably/i)) {
		service.supportedWEBM = true;
		service.supportedVideo.push("WEBM");
	}
	if(service.elVideoPlayer.canPlayType("video/mp4").match(/maybe|probably/i)) {
		service.supportedMP4 = true;
		service.supportedVideo.push("MP4");
	}
	
	/* playing */
	service.offset = 0;
	service.state = {};
	service.pl = {};
	
	/* set init state values or reset */
	service.resetStateAndPl = function() {
		service.state.show = false;
		service.state.playing = false;	
		service.state.file = null;
		service.state.position = 0;
		service.state.volume = Math.round(service.elAudioPlayer.volume * 100);
		
		service.state.format = null;
		service.state.formats = new Array();
		service.state.astream = null;
		service.state.astreams = new Array();
		service.state.sstream = null;
		service.state.sstreams = new Array();
		service.state.sstreams.push({idx:-1, type:'SUB', info:'subtitles off', language:'OFF', codec:null});
		
		service.pl.playlists = [];
		service.pl.playlist = {id: 0, name: '', items: []};
		service.pl.idx = 0;
		service.pl.random = false;
		service.pl.repeat = false;
		service.pl.repeatOne = false;
	};
	service.resetStateAndPl();
	
	/* settings */
	service.settings = {videoApproach: 'DEFAULT'};
	
	/* listener */
	service.refreshPosition = function(dingens) {
		var src = this; // video or audio element. alt: dingens.srcElement. src == service.elAudioPlayer
		$rootScope.$apply(function() {
			service.state.position = Number(service.offset) + Number(src.currentTime);
		});

		if(trackObject) {
			try {
				trackObject.activeCues.refreshCues.apply(trackObject.activeCues);
			} catch(error) {}
			captionator.rebuildCaptions(service.elVideoPlayer);
		}
	};
	service.elAudioPlayer.addEventListener('timeupdate',service.refreshPosition);
	service.elVideoPlayer.addEventListener('timeupdate',service.refreshPosition);

	service.onVolumeChange = function($event,refdivid) {
		var refdiv = document.getElementById(refdivid);
		var per = lmBase.mouseInDiv(refdiv,$event);
		
		service.elAudioPlayer.volume = per;
		service.elVideoPlayer.volume = per;
		service.state.volume = Math.round(per * 100);
	};
	
	service.onPositionChange = function($event,refdivid) {
		var refdiv = document.getElementById(refdivid);
		var newpos = lmBase.mouseInDiv(refdiv,$event) * service.state.file.duration;
		
		if('VIDEO' == service.state.file.type && service.state.format.transcoding) {
			service.offset = Math.round(newpos);
			service.refreshVideo(true);
		}
		else {
			service.elVideoPlayer.currentTime = newpos;
			service.elAudioPlayer.currentTime = newpos;
		}
	};
	
	service.refreshPlaying = function() {
		$rootScope.$apply(function() {
			service.state.playing = !(service.elVideoPlayer.paused && service.elAudioPlayer.paused);
		});
	};
	service.elVideoPlayer.addEventListener('playing',service.refreshPlaying);
	service.elAudioPlayer.addEventListener('playing',service.refreshPlaying);
	service.elVideoPlayer.addEventListener('pause',service.refreshPlaying);
	service.elAudioPlayer.addEventListener('pause',service.refreshPlaying);
	
	/* === Next track on end === */
	service.refreshEnded = function() {
		$rootScope.$apply(function() {
			if(service.pl.repeat && service.pl.repeatOne) service.playAgain();
			else service.playNext();
		});
	};
	service.elAudioPlayer.addEventListener('ended',service.refreshEnded);	
	service.elVideoPlayer.addEventListener('ended',service.refreshEnded);	
	
	
	/* actions internal */
	service.beforePlay = function() {
		service.elAudioPlayer.pause();
		service.elVideoPlayer.pause();
		try { service.elAudioPlayer.currentTime = 0; } catch(error) {};
		try { service.elVideoPlayer.currentTime = 0; } catch(error) {};
		service.offset = 0;
		service.state.format = null;
		service.state.formats = new Array();
		service.state.astream = null;
		service.state.astreams = new Array();
		service.state.sstream = null;
		service.state.sstreams.splice(1,service.state.sstreams.length - 1);
		service.disableSubtitle();
	};
	
	service.playAudio = function(valueFile) {
		var supportedStandard = null;
		for(x in valueFile.formats) {
			var af = valueFile.formats[x];
			for(y in service.supportedAudio) {
				var supForm = service.supportedAudio[y];
				if(supForm == af.standard && supportedStandard == null) {
					supportedStandard = af.standard;
				}
			}
		}
		
		var playurl = lmBase.url + 'f/streamAudio/' + valueFile.id +'/x' + '?standard=' + (supportedStandard == null ? 'WEBM_VORBIS' : supportedStandard);
		
		
		service.elAudioPlayer.src = playurl;
		service.elAudioPlayer.play();		
	}
	
	service.playVideo = function(valueFile) {
		/* extract audio and sub streams */
		for(x in valueFile.formats) {
			var format = valueFile.formats[x];
			if(format.original) {
				for(y in format.streams) {
					var stream = format.streams[y];
					if("AUDIO" == stream.type) {
						service.state.astreams.push(stream);
					}
					if("SUB" == stream.type && ('SUBRIP' == stream.codec || 'WEBVTT' == stream.codec)) {
						service.state.sstreams.push(stream);
					}
				}
				if(service.state.astreams.length > 0) {
					service.state.astream = service.state.astreams[0];
				}
			}
		}
		
		/* add transcoding streams */ 
		if(service.supportedWEBM) {
			for(tfi in service.transcodingFormats) {
				var tf = service.transcodingFormats[tfi];
				var added = false;
				for(x in valueFile.formats) {
					var format = valueFile.formats[x];
					if('WEBM' == format.standard && format.layout == tf.layout) {
						service.state.formats.push(format);
						if(service.state.format == null) {
							service.state.format = format;
						}
						added = true;
					}
				}
				if(!added) {
					service.state.formats.push(tf);
				}
			}
		}
		
		/* add available playable streams */
		for(x in valueFile.formats) {
			var format = valueFile.formats[x];
			if((service.supportedWEBM && 'WEBM' != format.standard) || format.layout == null) {
				service.state.formats.push(format);
			}
		}
		
		/* add guinames and metainfos */
		for(x in service.state.formats) {
			var format = service.state.formats[x];
			format.guiname = (format.layout == null ? 'Custom' : service.layoutMap[format.layout]);
			format.guiname += ' ' + format.mime.substring(6);

			if(format.original) format.guiname += ' (original)';
			else if(format.transcoding) format.guiname += ' (transcode)';
			
			/* add nicesize and urls, and playable */
			format.url = lmBase.url + 'f/streamVideo/' + valueFile.id + '/' + valueFile.urlname;
			if(!format.original) {
				format.url += '.' + format.standard + '?standard=' + format.standard + '&layout=' + format.layout + '&sau=666';
			}
		}		
		
		/* select format to play. Approaches: ['DEFAULT','P360','P720','P1080'] */
		if('P360' == service.settings.videoApproach || 'P720' == service.settings.videoApproach || 'P1080' == service.settings.videoApproach) {
			for(x in service.state.formats) {
				var format = service.state.formats[x];
				if(service.settings.videoApproach == format.layout && service.isStandardSupported(format.standard)) {
					if(service.state.format == null) {
						service.state.format = format;
					}
					else if(service.state.format.transcoding && !format.transcoding) {
						service.state.format = format;
					}
				}	
			}
		}
		else {
			for(x in service.state.formats) {
				var format = service.state.formats[x];
				if(service.isStandardSupported(format.standard)) {
					if(service.state.format == null) {
						service.state.format = format;
					}
					else if(service.state.format.transcoding && !format.transcoding) {
						service.state.format = format;
					}
				}
			}
		}
		
		/* no suitable format --> notification */
		if(service.state.format == null) {
			if(service.supportedWEBM) {
				serviceNotification.error('MSG_VIDEO_TRY_WEBM');
			}
			else {
				serviceNotification.error('MSG_VIDEO_UNSUPPORTED');
			}
		}
		else {
			service.refreshVideo();
			// service.elVideoScrollPoint.scrollIntoView();
		}		
	}
	
	service.refreshVideo = function(sametime) {
//		$scope.playerShow = true;
		
		if(sametime && service.state.format.transcoding) {
			service.offset += Math.floor(Number(service.elVideoPlayer.currentTime));
		}
		else {
			service.offset = 0;
		}
		
		var playurl = lmBase.url + 'f/streamVideo/' + service.state.file.id + '/x';
		if(!service.state.format.original) {
			playurl += '?standard=' + service.state.format.standard + '&layout=' + service.state.format.layout;
		}
		
		if(service.state.format.transcoding) {
			if(service.offset != 0) {
				playurl += "&ss=" + service.offset;
			}
			if(service.state.astream != null) {
				playurl += "&sau=" + service.state.astream.idx;
			}
		}
		service.elVideoPlayer.src = playurl;
		service.elVideoPlayer.play();
		
		service.refreshSub();
	};
	
	service.refreshSub = function() {
		if(service.state.sstream) {
			var earl = lmBase.url + 'f/streamVideoSub/' + service.state.file.id + '/' + service.state.sstream.idx;
			
			if(service.offset != 0) {
				earl += "?ss=" + service.offset;
			}
			
			trackObject = new captionator.TextTrack(captionator.generateID(),'subtitles','blub','de',earl,null);
			trackObject.videoNode = service.elVideoPlayer;
			trackObject.mode = captionator.TextTrack.SHOWING;
		}
		else {
			service.disableSubtitle();
		}
		
		/*		
			window.addEventListener("resize", function(eventData) {
				videoElement._captionator_dirtyBit = true; // mark video as dirty, force captionator to rerender captions
				captionator.rebuildCaptions(videoElement);
			},false);
		*/
	};
	
	service.disableSubtitle = function() {
		if(trackObject) {
			trackObject.mode = captionator.TextTrack.OFF;
			captionator.rebuildCaptions(service.elVideoPlayer);
			trackObject = null;			
		}
	}	
	
	/* helper methods */
	service.isStandardSupported = function(valueStandard) {
		if(valueStandard == null || valueStandard == '') {
			return false;
		}
		for(x in service.supportedVideo) {
			if(valueStandard == service.supportedVideo[x]) {
				return true;
			}
		}
	};	
	
	/* actions */
	service.playIndex = function(valueIndex) {
		if(service.pl.playlist.items.length == 0 || valueIndex < 0) {
			return;
		}
		
		if(valueIndex >= service.pl.playlist.items.length) {
			if(service.pl.repeat && !service.pl.repeatOne) valueIndex = 0;
			else return;
		}

		var fileToPlay = service.pl.playlist.items[valueIndex];
		service.state.file = fileToPlay;		
		service.pl.idx = valueIndex;
		
		/* Loading files with details (formats, ...) initially is expensive. Therefore we lazy-load details here as soon as required. */
		if(fileToPlay.formats === undefined || fileToPlay.formats.length == 0) {
			$http.get(lmBase.url + 'f/file/' + fileToPlay.id)
				.success(function(data) {
					fileToPlay.formats = data.formats;
					service.playOnDetailsAvailable(fileToPlay);
				}).error(serviceLimboMedia.handleHttpError);
		} else {
			service.playOnDetailsAvailable(fileToPlay);
		}
	};
	
	/* Called directly or after reloading file details (formats, ...). */
	service.playOnDetailsAvailable = function(valueFile) {
		service.beforePlay();
		
		if(service.state.file.type == 'AUDIO') {
			service.playAudio(valueFile);
		}
		else if(service.state.file.type == 'VIDEO') {
			service.playVideo(valueFile);
		}
	};
	
	service.stop = function() {
		service.beforePlay();
	};
	
	service.playPause = function() {
		if(service.state.file == null) return;
		var el = 'VIDEO' == service.state.file.type ? service.elVideoPlayer : service.elAudioPlayer;
		if (el.paused || el.ended) el.play();
		else el.pause();
	};
	
	service.changeFormat = function(format) {
		service.state.format = format;
		service.refreshVideo(true);
	};
	
	service.changeAudioStream = function(as) {
		service.state.astream = as;
		service.refreshVideo(true);
	};
	
	service.changeSubStream = function(ss) {
		service.state.sstream = ss.idx == -1 ? null : ss;
		service.refreshSub();
	};		
	
	service.playReset = function() {
		service.playIndex(service.pl.idx);
	};
	
	service.playRandom = function() {
		service.playIndex(Math.floor(Math.random() * service.pl.playlist.items.length));
	};
	
	service.playAgain = function() {
		service.playIndex(service.pl.idx);
	}
	
	service.playPrev = function() {
		if(service.pl.random) service.playRandom();
		else service.playIndex(service.pl.idx - 1);
	};
	
	service.playNext = function() {
		if(service.pl.random) service.playRandom();
		else service.playIndex(service.pl.idx + 1);
	};
	
	service.playFile = function(valueFile) {
		service.pl.playlist.items = [valueFile];
		service.playIndex(0);
		service.playlistOnChange();
		serviceSidebar.setShow(true);
	};
	
	service.playFiles = function(valueFiles) {
		service.pl.playlist.items = new Array();
		for(x in valueFiles) {
			service.pl.playlist.items.push(valueFiles[x]);
		}
		service.playIndex(0);
		service.playlistOnChange();
		serviceSidebar.setShow(true);
	};	
	
	service.addFile = function(valueFile) {
		service.pl.playlist.items.push(valueFile);
		service.playlistOnChange();
	};

	service.addFiles = function(valueFiles) {
		for(x in valueFiles) {
			service.pl.playlist.items.push(valueFiles[x]);
		}
		service.playlistOnChange();
	};
	
	service.toggleFullscreen = function() {
		serviceFullscreen.onToggleFullscreen(service.elVideoFrame);
	};
	
	/* playlist stuff */
	service.playlistsLoad = function(initial) {
		$http.get(lmBase.url + 'f/playlists/list')
			.success(function(data) {
				service.pl.playlists = data;
				if(initial) {
					for(y in data) {
						var pl = data[y];
						if('last' == pl.name) {
							service.playlistLoad(pl,false);
						}
					}
				}
			}).error(serviceLimboMedia.handleHttpError);
	};
	
	service.playlistLoad = function(playlist,playit) {
		$http.get(lmBase.url + 'f/playlist/' + playlist.id)
			.success(function(data) {
				for(x in data.items) {
					lmBase.pimpLmFile(data.items[x]);
				}
				service.pl.playlist = data;
				if(playit) service.playIndex(0);
			}).error(serviceLimboMedia.handleHttpError);
	};	
	
	service.playlistSave = function() {
		var req = {name: service.pl.playlist.name, items: lmBase.arrayToIdArray(service.pl.playlist.items)};
		$http.post(lmBase.url + 'f/playlist',req)
			.success(function(data) {
				serviceNotification.ok('MSG_SAVED');
				service.playlistsLoad(false);
			}).error(serviceLimboMedia.handleHttpError);
	};

	service.playlistSaveLast = function() {
		var req = {name: 'last', items: lmBase.arrayToIdArray(service.pl.playlist.items)};
		$http.post(lmBase.url + 'f/playlist',req)
			.success(function(data) {
			}).error(serviceLimboMedia.handleHttpError);
	};
	
	service.playlistDelete = function(playlist) {
		$http.delete(lmBase.url + 'f/playlist/' + playlist.id)
		.success(function(data) {
			service.pl.playlist = {id: 0, name: '', items: []};
			service.playlistsLoad();
		})
		.error(serviceLimboMedia.handleHttpError);
	};
	
	service.playlistOnChange = function() {
		if(serviceLimboMedia.hasRole('ROLE_WRITE')) {
			service.playlistSaveLast();
		}
	};	
	
	$rootScope.$on('lmLogin', function() {
		service.settings = serviceLimboMedia.getRemoteSettings();
		if(service.settings.videoApproach == null) service.settings.videoApproach = 'DEFAULT';

		service.playlistsLoad(true);
	});
	
	$rootScope.$on('lmLogout', function() {
		service.beforePlay();
		service.resetStateAndPl();
	});
	
	return service;
}]);

