lm.factory('serviceUpload', [ '$rootScope', '$http', 'serviceNotification' , function($rootScope, $http, serviceNotification) {
	var service = new Object();
	
	/* attributes */
	service.queue = new Array();
	
	/* upload complete listener */
	service.listener = null;
	service.registerListener = function(valueListener) {
		service.listener = valueListener;
	};
	
	/* show upload state */
	service.state = new Object();
	service.state.uploading = false;
	service.state.percent = 0;
	service.state.currentItem = null;
	service.state.fileCounter = 0;
	
	/* === Hide-Show === */
	service.state.show = false;
	service.setShowState = function(valueShow) {
		if(valueShow && !service.state.show) {
			window.scrollTo(0,0);
		}
		service.state.show = valueShow;
	};
	service.toggleShow = function() {
		service.setShowState(!service.state.show);
	};	
	
	/* functions */
	service.uploadFiles = function(valueFiles, valueURL, valueForm, valueParams) {
		for(x in valueFiles) {
			var fi = valueFiles[x];
			
			var item = new Object();
			item.file = fi;
			item.url = valueURL;
			item.params = valueParams;
			item.form = valueForm;
			
			service.queue.push(item);
		}
		
		/* Show info about uploading */
		service.setShowState(true);
		
		/* Start upload */
		service.next(true);
	};
	
	service.putNextToWorker = function(fromWithinScope) {
		if(fromWithinScope) {
			service.state.currentItem = service.queue.shift();
			service.state.uploading = true;
		}
		else {
			$rootScope.$apply(function() {
				service.state.currentItem = service.queue.shift();
				service.state.uploading = true;
			});
		}
		
	};
	
	service.next = function(fromWithinScope) {
		if(service.state.currentItem != null) {
			return;
		}
	
		if(service.queue.length == 0) {
			return;
		}
		
		service.putNextToWorker(fromWithinScope);
	
		/* Create XHR */
		var xhr = new XMLHttpRequest();
		
		/* Add listeners */
		xhr.upload.addEventListener("progress", service.uploadProgress, false);
		xhr.addEventListener("load", service.uploadEvent, false);
		xhr.addEventListener("error", service.uploadEvent, false);
		xhr.addEventListener("abort", service.uploadEvent, false);
		
		/* Direct upload (form upload not used anymore) */
		xhr.open("POST", service.state.currentItem.url);
		xhr.setRequestHeader("X-LM-filename", lmBase.cutToLatin1(service.state.currentItem.file.name));
		xhr.setRequestHeader("LM-csrf","yeah");
		if(service.state.currentItem.params != null) {
			for(var x in service.state.currentItem.params) {
				xhr.setRequestHeader('X-LM-' + x, service.state.currentItem.params[x]);
			}
		}			
		xhr.send(service.state.currentItem.file);
				
	};
	
	service.uploadProgress = function(evt) {
		if (evt.lengthComputable) {
			var percentComplete = Math.round(evt.loaded * 100 / evt.total);
			$rootScope.$apply(function() {
				service.state.percent = percentComplete;
			});
		}
	};
	
	service.uploadEvent = function(evt) {
		if(200 != this.status) {
			var response = this.response;
			$rootScope.$apply(function() {
				serviceNotification.httpError(response);
			});
		}
		
		service.uploadDone();
	};
	
	service.uploadDone = function() {
		service.state.currentItem = null;
		$rootScope.$apply(function() {
			service.state.uploading = false;
			service.state.percent = 0;
			service.state.fileCounter++;
			
			if(service.queue.length == 0) {
				service.setShowState(false);
				serviceNotification.ok('MSG_UPLOAD_OK');
				service.state.fileCounter = 0;
				
				if(service.listener != null) {
					service.listener();
				}
			}
		});
		service.next(false);
	};
	
	return service;
}]);
